{============================================================================}
{  FontComps.pas
   Version 0.99.2 beta
   Last Modified 02-16-2000
   Copyright Sean Newton, 2000
   No Warranty!



   See FontComps.txt for more information.
}
{============================================================================}


unit FontComps;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons;



{============================================================================}
{ TColorCombo }

type
  TColorCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FRichEdit: TRichEdit;
    FColor:integer;
    FReturnFocus: boolean;
    FColorClick:TNotifyEvent;
    FColorChange:TNotifyEvent;
    procedure MeasureItem(Control: TWinControl; Index: Integer;var Height: Integer);
    procedure DrawItem(Control: TWinControl; Index: Integer; Rect: TRect;State: TOwnerDrawState);
    procedure DoClick(Sender:TObject);
  protected
    { Protected declarations }
    procedure SetItems;
    property Items;
    property ItemHeight;
    property Style;
    property OnDrawItem;
    property OnMeasureItem;
    property OnClick;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Update;
    procedure Init;
  published
    { Published declarations }
    property Color;
    property Ctl3D;
    property Height;
    property RichEdit: TRichEdit  read FRichEdit write FRichEdit;
    property FontColor: integer read FColor write FColor;
    property ReturnFocus:boolean read FReturnFocus write FReturnFocus;
    property OnDropDown;
    property OnEnter;
    property OnExit;
    property OnColorClick: TNotifyEvent read fColorClick write fColorClick;
    property OnColorChange: TNotifyEvent read fColorChange write fColorChange;
  end;

{============================================================================}
{ TFontCombo }

type
  TFontCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FFontName: string;
    FRichEdit: TRichEdit;
    FDraw: Boolean;
    FFontChange: TNotifyEvent;
    FFontClick: TNotifyEvent;
    FReturnFocus: boolean;
    procedure MeasureItem(Control: TWinControl; Index: Integer;var Height: Integer);
    procedure DrawItem(Control: TWinControl; Index: Integer; Rect: TRect;State: TOwnerDrawState);
    procedure DoClick(Sender: TObject);
    procedure SetDraw(Value:boolean);
  protected
    { Protected declarations }
    property Items;
    property ItemHeight;
    property Style;
    property OnDrawItem;
    property OnMeasureItem;
    property OnClick;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Init;
    procedure Update;
  published
    { Published declarations }
    property Color;
    property Ctl3D;
    property Height;
    property RichEdit: TRichEdit read FRichEdit write FRichEdit;
    property Fontname: string read FFontName write FFontName;
    property DrawFonts:boolean read FDraw write SetDraw;
    property ReturnFocus:boolean read FReturnFocus write FReturnFocus;
    property OnFontChange:TNotifyEvent read FFontChange write FFontChange;
    property OnFontClick:TNotifyEvent read FFontClick write FFontClick;
    property OnDropDown;
    property OnEnter;
    property OnExit;
  end;

{============================================================================}
{ TSizeCombo }


type
  TSizeCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FRichEdit: TRichEdit;
    OldSize,
    FFontSize: integer;
    FSizeChange: TNotifyEvent;
    FSizeClick: TNotifyEvent;
    FReturnFocus: boolean;
  protected
    { Protected declarations }
    procedure DoClick(Sender: TObject);
    procedure DoChange(Sender: TObject);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Init;
    procedure Update;
  published
    { Published declarations }
    property RichEdit: TRichEdit read FRichEdit write FRichEdit;
    property FontSize: integer read FFontSize write FFontSize;
    property ReturnFocus:boolean read FReturnFocus write FReturnFocus;
    property OnSizeChange: TNotifyEvent read fSizeChange write fSizeChange;
    property OnSizeClick: TNotifyEvent read fSizeClick write fSizeClick;
    property OnDropDown;
    property OnEnter;
    property OnExit;
  end;

{============================================================================}
{ TBoldButton }

type
  TBoldButton = class(TSpeedButton)
    private
      { Private declarations }
      FRichEdit: TRichEdit;
      FReturnFocus: Boolean;
      FBold: boolean;
    public
      constructor Create(AOwner:TComponent); override;
      procedure Update;
      procedure BoldClick(Sender: TObject);
    published
      property RichEdit: TRichEdit read FRichEdit write FRichEdit;
      property ReturnFocus: Boolean read FReturnFocus write FReturnFocus;
      property Bold: boolean read FBold write FBold;
  end;

{============================================================================}
{ TItalicButton }

type
  TItalicButton = class(TSpeedButton)
    private
      { Private declarations }
      FRichEdit: TRichEdit;
      FReturnFocus: Boolean;
      FItalic: boolean;
    public
      constructor Create(AOwner:TComponent); override;
      procedure Update;
      procedure ItalicClick(Sender: TObject);
    published
      property RichEdit: TRichEdit read FRichEdit write FRichEdit;
      property ReturnFocus: Boolean read FReturnFocus write FReturnFocus;
      property Italic: boolean read FItalic write FItalic;
  end;

{============================================================================}
{ TUnderlineButton }

type
  TUnderlineButton = class(TSpeedButton)
    private
      { Private declarations }
      FRichEdit: TRichEdit;
      FReturnFocus: Boolean;
      FUnderline: boolean;
    public
      constructor Create(AOwner:TComponent); override;
      procedure Update;
      procedure UnderlineClick(Sender: TObject);
    published
      property RichEdit: TRichEdit read FRichEdit write FRichEdit;
      property ReturnFocus: Boolean read FReturnFocus write FReturnFocus;
      property Underline: Boolean read FUnderline write FUnderline;
  end;


procedure Register;

implementation

{============================================================================}
{ TColorCombo }
{ }
{----------------------------------------------------------------------------}


procedure TColorCombo.Update;
begin
  If RichEdit <> nil then
  begin
    { SelAttributes }
    FontColor := RichEdit.SelAttributes.Color;
    ItemIndex := Items.IndexOf(IntToStr(FontColor));
  end;
end;

procedure TColorCombo.SetItems;
begin
  Items[0] := IntToStr(clBlack);
  Items[1] := IntToStr(clMaroon);
  Items[2] := IntToStr(clGreen);
  Items[3] := IntToStr(clOlive);
  Items[4] := IntToStr(clNavy);
  Items[5] := IntToStr(clPurple);
  Items[6] := IntToStr(clTeal);
  Items[7] := IntToStr(clGray);
  Items[8] := IntToStr(clSilver);
  Items[9] := IntToStr(clRed);
  Items[10] := IntToStr(clLime);
  Items[11] := IntToStr(clBlue);
  Items[12] := IntToStr(clFuchsia);
  Items[13] := IntToStr(clAqua);
  Items[14] := IntToStr(clWhite);
end;

procedure TColorCombo.DoClick(Sender:TObject);
begin
  FontColor := StrToInt(Items[ItemIndex]);
  if RichEdit <> nil then
  begin
    RichEdit.SelAttributes.Color := StrToInt(Items[ItemIndex]);
    If ReturnFocus then RichEdit.SetFocus;
  end;
  If Assigned(OnColorClick) then OnColorClick(Sender);
end;

procedure TColorCombo.DrawItem(Control: TWinControl; Index: Integer; Rect: TRect;State: TOwnerDrawState);
begin
  Inc(Rect.left,2);
  Inc(Rect.top,2);
  Dec(Rect.Bottom,2);
  Dec(Rect.right,2);

  with Canvas do
  begin
    Brush.Color := StrToInt(Items[Index]);
    FillRect(Rect);
  end;
end;

procedure TColorCombo.MeasureItem(Control: TWinControl; Index: Integer;var Height: Integer);
begin
  with Canvas do
  begin
    Height := 20; // measure ascenders and descenders
  end;
end;

procedure TColorCombo.Init;
begin
  SetItems;
  ItemIndex := 0;
  Update;
end;

constructor TColorCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ItemHeight := 20;
  Style := csOwnerDrawFixed;
  OnDrawItem := DrawItem;
  OnMeasureItem := MeasureItem;
  OnClick := DoClick;
end;

{============================================================================}
{ TFontCombo }
{ }
{----------------------------------------------------------------------------}


procedure TFontCombo.Update;
begin
  If RichEdit <> nil then
  begin
    { SelAttributes }
    FontName := RichEdit.SelAttributes.Name;
    ItemIndex := Items.IndexOf(FontName);
  end;
end;

procedure TFontCombo.SetDraw(Value:boolean);
begin
  If Value then
  begin
    Style := csOwnerDrawFixed;
  end
  else
  begin
    Style := csDropDownList;
  end;
  FDraw := Value;
end;

procedure TFontCombo.DrawItem(Control: TWinControl; Index: Integer; Rect: TRect;State: TOwnerDrawState);
begin
  with Canvas do
  begin
    FillRect(Rect);
    Font.Name := Items[Index];
    Font.Size := 0;    // use font's preferred size
    TextOut(Rect.Left+1, Rect.Top+1, Items[Index]);
  end;
end;

procedure TFontCombo.MeasureItem(Control: TWinControl; Index: Integer;var Height: Integer);
begin
  with Canvas do
  begin
    Font.Name := Items[Index];
    Font.Size := 0;                 // use font's preferred size
    Height := TextHeight('Wg') + 2; // measure ascenders and descenders
  end;
end;

procedure TFontCombo.Init;
var
  TempFonts: TStringList;
begin
  TempFonts := TStringList.Create;
  TempFonts.AddStrings(Screen.Fonts);
  TempFonts.Sort;
  Items := TempFonts;
  TempFonts.Free;
  ItemIndex := 0;
  Update;
end;

procedure TFontCombo.DoClick(Sender: TObject);
begin
  FFontName := Items[ItemIndex];
  if RichEdit <> nil then
  begin
    RichEdit.SelAttributes.Name := Items[ItemIndex];
    If ReturnFocus then RichEdit.SetFocus;
  end;
  If Assigned(OnFontClick) then OnFontClick(Sender);
end;

constructor TFontCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ItemHeight := 20;
  OnDrawItem := DrawItem;
  OnMeasureItem := MeasureItem;
  OnClick := DoClick;
  OnChange := OnFontChange;
end;

{============================================================================}
{ TSizeCombo }
{ }
{----------------------------------------------------------------------------}


procedure TSizeCombo.Update;
begin
  If RichEdit <> nil then
  begin
    { SelAttributes }
    FontSize := RichEdit.SelAttributes.Size;
    Text := IntToStr(FontSize);
    If Items.IndexOf(IntToStr(FontSize)) <> -1 then
      ItemIndex := Items.IndexOf(IntToStr(FontSize));
  end;
end;

procedure TSizeCombo.DoClick(Sender: TObject);
begin
  if RichEdit <> nil then
  begin
    RichEdit.SelAttributes.Size := StrToInt(Items[ItemIndex]);
    If ReturnFocus then RichEdit.SetFocus;
  end;
  If Assigned(OnSizeClick) then OnSizeClick(Sender);
  OldSize := FontSize;
end;

procedure TSizeCombo.DoChange(Sender:TObject);
begin
  try
    FontSize := StrToInt(Text)
  except
    FontSize := OldSize;
    DoClick(Sender);
  end;
  OldSize := FontSize;
  If Assigned(OnSizeChange) then OnSizeChange(Sender);
  If RichEdit <> nil then
  begin
    RichEdit.SelAttributes.Size := StrToInt(Text);
    If ReturnFocus then RichEdit.SetFocus;
  end;
end;

procedure TSizeCombo.Init;
const
  Delimit = #13;
var
  Counter : integer;
begin
  For Counter := 0 to 10 do
    Items[Counter] := IntToStr(8+(2*Counter));
  Items[11] := '36';
  Items[12] := '48';
  Items[13] := '72';
  ItemIndex := 0;
  Update;
end;

constructor TSizeCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Text := '';
  ItemHeight := 20;
  Height := 26;
  Style := csDropDown;
  OnClick := DoClick;
  OnChange := DoChange;
  OldSize := FontSize;
end;

{============================================================================}
{ TBoldButton }
{----------------------------------------------------------------------------}

constructor TBoldButton.Create(AOwner:TComponent);
var
  NewBitmap :  TBitmap;
begin
  inherited Create(AOwner);

  AllowAllUp := True;
  GroupIndex := 522;

  Bold := False;

  NewBitmap := TBitmap.Create;
  NewBitmap.Height := 16;
  NewBitmap.Width := 16;
  NewBitmap.Canvas.Font.Size := 10;
  NewBitmap.Canvas.Font.Style := [fsBold];
  NewBitmap.Canvas.TextOut(2,2,'B');
  Glyph := NewBitmap;

  OnClick := BoldClick;
end;

procedure TBoldButton.BoldClick(Sender: TObject);
var
  FontStyle: TFontStyles;
begin
  Bold := not(Bold);
  If RichEdit <> nil then
  begin
    FontStyle := RichEdit.SelAttributes.Style;
    If fsBold in FontStyle then
      Exclude(FontStyle, fsBold)
    else
      Include(FontStyle, fsBold);
    RichEdit.SelAttributes.Style := FontStyle;
  end;
  If Bold then Down := True else Down := False;
  Update;
  If ReturnFocus then if RichEdit <> nil then RichEdit.SetFocus;
end;

procedure TBoldButton.Update;
begin
  If RichEdit <> nil then
  begin
    If fsBold in RichEdit.SelAttributes.Style then
    begin
      Down := True;
      Font.Color := clBlack;
      Font.Size := 12;
    end
    else
    begin
      Down := False;
      Font.Color := clGray;
      Font.Size := 10;
    end;
  end;
  Bold := not(Down);
end;

{============================================================================}
{ TItalicButton }
{----------------------------------------------------------------------------}

constructor TItalicButton.Create(AOwner:TComponent);
var
  NewBitmap : TBitmap;
begin
  inherited Create(AOwner);

  Italic := False;

  AllowAllUp := True;
  GroupIndex := 523;

  NewBitmap := TBitmap.Create;
  NewBitmap.Height := 16;
  NewBitmap.Width := 16;
  NewBitmap.Canvas.Font.Size := 10;
  NewBitmap.Canvas.Font.Style := [fsItalic];
  NewBitmap.Canvas.TextOut(2,2,'i');
  Glyph := NewBitmap;

  OnClick := ItalicClick;
end;

procedure TItalicButton.ItalicClick(Sender: TObject);
var
  FontStyle: TFontStyles;
begin
  Italic := not(Italic);
  If RichEdit <> nil then
  begin
    FontStyle := RichEdit.SelAttributes.Style;
    If fsItalic in FontStyle then
      Exclude(FontStyle, fsItalic)
    else
      Include(FontStyle, fsItalic);
    RichEdit.SelAttributes.Style := FontStyle;
  end;
  If Italic then Down := True else Down := False;
  Update;
  If ReturnFocus then if RichEdit <> nil then RichEdit.SetFocus;
end;

procedure TItalicButton.Update;
begin
  If RichEdit <> nil then
  begin
    If fsItalic in RichEdit.SelAttributes.Style then
    begin
      Font.Color := clBlack;
      Font.Size := 12;
      Down := True;
    end
    else
    begin
      Font.Color := clGray;
      Font.Size := 10;
      Down := False;
    end;
  end;
  Italic := not(Down);
end;

{============================================================================}
{ TUnderlineButton }
{----------------------------------------------------------------------------}

constructor TUnderlineButton.Create(AOwner:TComponent);
var
  NewBitmap : TBitmap;
begin
  inherited Create(AOwner);

  AllowAllUp := True;
  GroupIndex := 524;

  Underline := False;

  NewBitmap := TBitmap.Create;
  NewBitmap.Height := 16;
  NewBitmap.Width := 16;
  NewBitmap.Canvas.Font.Size := 10;
  NewBitmap.Canvas.Font.Style := [fsUnderline];
  NewBitmap.Canvas.TextOut(2,2,'u');
  Glyph := NewBitmap;

  OnClick := UnderlineClick;
end;

procedure TUnderlineButton.UnderlineClick(Sender: TObject);
var
  FontStyle: TFontStyles;
begin
  Underline := not(Underline);
  If RichEdit <> nil then
  begin
    FontStyle := RichEdit.SelAttributes.Style;
    If fsUnderline in FontStyle then
      Exclude(FontStyle, fsUnderline)
    else
      Include(FontStyle, fsUnderline);
    RichEdit.SelAttributes.Style := FontStyle;
  end;
  If Underline then Down := True else Down := False;
  Update;
  If ReturnFocus then if RichEdit <> nil then RichEdit.SetFocus;
end;


procedure TUnderlineButton.Update;
begin
  If RichEdit <> nil then
  begin
    If fsUnderline in RichEdit.SelAttributes.Style then
    begin
      Font.Size := 12;
      Font.Color := clBlack;
      Down := True;
    end
    else
    begin
      Down := False;
      Font.Color := clGray;
      Font.Size := 10;
    end;
  end;
  Underline := not(Down);
end;


{============================================================================}
{ Register }
{----------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('Font', [TColorCombo]);
  RegisterComponents('Font', [TFontCombo]);
  RegisterComponents('Font', [TSizeCombo]);
  RegisterComponents('Font', [TBoldButton]);
  RegisterComponents('Font', [TItalicButton]);
  RegisterComponents('Font', [TUnderlineButton]);
end;

end.
